@@ -4,11 +4,12 @@ import (
 	"testing"
 
 	"github.com/walles/moar/twin"
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 )
 
 func TestParseScrollHint(t *testing.T) {
-	token := parseScrollHint("ESC[7m>", nil)
+	token, err := parseScrollHint("ESC[7m>")
+	assert.NilError(t, err)
 	assert.Equal(t, token, twin.Cell{
 		Rune:  '>',
 		Style: twin.StyleDefault.WithAttr(twin.AttrReverse),
